<?php
// $Id: class.parse.php,v 1.0.0 2006/07/08 00:00:00 TK Exp $


class Convert_Cache_Parse extends Convert_Cache
{
	var $page;

	var $nocache = false;

	var $block_regex;

	var $block_plugin = array();


	function Convert_Cache_Parse($page)
	{
		$this->page = $page;

		if (PKWKEXP_DISABLE_MULTILINE_PLUGIN_HACK) {
			$this->block_regex = '/^#([^\(]+)(?:\((.*)\))?/';
		} else {
			$this->block_regex = '/^#([^\(\{]+)(?:\(([^\r]*)\))?(\{*)/';
		}

		if (! file_exists(CONVERT_CACHE_DIR . 'plugin.cfg')) {
			$this->makeConfigCache(array('plugin'));
		}
		$config = $this->unserializeFile(CONVERT_CACHE_DIR . 'plugin.cfg');
		$block_plugin = array();
		foreach ($config as $array) {
			list($name, $rule)   = $array;
			$block_plugin[$name] = $rule;
		}
		$this->block_plugin = $block_plugin;
	}

	function run()
	{
		$this->parse($this->getLines());
		$this->makeLog();
	}

	function getLines()
	{
		$data  = $this->getContents(get_filename($this->page));
		$lines = explode("\n", $data);

		if (PKWKEXP_DISABLE_MULTILINE_PLUGIN_HACK) {
			return $lines;
		}

		$return_lines = $matches = array();
		while (! empty($lines)) {
			$line = array_shift($lines);

			if (substr($line, 0, 1) == '#' &&
			    preg_match('/^#[^{]+(\{\{+)\s*$/', $line, & $matches)) {
				$len   = strlen($matches[1]);
				$line .= "\r";// Delimiter
				while (! empty($lines)) {
					$next_line = array_shift($lines);
					if (preg_match('/\}{' . $len . '}/', $next_line)) {
						$line .= $next_line;
						break;
					} else {
						$line .= $next_line . "\r";// Delimiter
					}
				}
			}
			$return_lines[] = $line;
		}
		return $return_lines;
	}

	function parse($lines)
	{
		while (! empty($lines)) {
			$line = rtrim(array_shift($lines), "\r\n");
			if (substr($line, 0, 2) == '//' &&
			    preg_match('#^//[ \t]*nocache$#', $line)) {
				$this->nocache = true;
				return;
			}
			if ($line == '' || $line{0} != '#') {
				continue;
			}
			$matches = array();
			if (preg_match($this->block_regex, $line, $matches)) {
				$this->goDiv($line, $matches);
			}
		}
	}

	function goDiv($line, $matches)
	{
		if (PKWKEXP_DISABLE_MULTILINE_PLUGIN_HACK) {
			$this->Div($matches);
		} else {
			$len  = strlen($matches[3]);
			if ($len === 0) {
				$this->Div($matches);
				return;
			}
			$regex = '/\{{' . $len . '}\s*\r(.*)\r\}{' . $len . '}/';
			if (preg_match($regex, $line, & $body)) { 
				$matches[2] .= "\r" . $body[1] . "\r";
				$this->Div($matches);
			}
		}
	}

	function Div($out)
	{
		list(, $name, $param) = array_pad($out, 3, null);

		if (array_key_exists($name, $this->block_plugin)) {
			$this->plugin_convert($name, explode(',' ,$param));
		}
	}

	function plugin_convert($name, $param)
	{
		$args  = explode('_', $this->block_plugin[$name]);
		$num   = array_shift($args);
		$array = array();
		foreach ($args as $arg) {
			$array[$arg] = true;
		}

		$filename = CONVERT_CACHE_LIB_DIR . 'plugin/' . $name . '.inc.php';
		if (file_exists($filename)) {
			require_once $filename;
		}
		$function = 'plugin_' . $name . '_cache';
		if (function_exists($function)) {
			$get_page = $function($this->page, $param, 'pagename');
		} else {
			$get_page = $this->plugin_pagename($param, $num);
		}

		if (isset($array['no'])) {
			$this->nocache = true;
			return;
		}
		if (isset($array['every'])) {
			$this->plugin_convert_every($get_page);
			return;
		}
		if (isset($array['inc'])) {
			$this->plugin_convert_inc($get_page);
			return;
		}
		if (isset($array['dir'])) {
			$slash = (isset($array['slash']));
			$this->plugin_convert_dir($get_page, $slash);
			return;
		}
	}

	function plugin_convert_every()
	{
		$this->log[] = $this->page . ' => every';
		$filename = CONVERT_CACHE_DIR . 'log/every';
		if (file_exists($filename)) {
			$array = $this->unserializeFile($filename);
		}
		$array[$this->page] = '';
		$this->putContents($filename, serialize($array));
	}

	function plugin_convert_inc($get_page)
	{
		$this->log[] = $get_page . ' => del';
	}

	function plugin_convert_dir($get_page, $slash)
	{
		$this->log[] = $get_page . ' => dir';
	}

	function plugin_pagename($param, $num = 1)
	{
		if (isset($param[--$num])) {
			return get_fullname(strip_bracket($param[$num]), $this->page);
		} else {
			return $this->page;
		}
	}

	function makeLog()
	{
		if ($this->nocache) {
			$filename = $this->getFileName($this->page, 'log', 'no');
			$this->putContents($filename, 'nocache');
			return;
		}

		$filename = $this->getFileName($this->page, 'log', 'log');
		if (file_exists($filename)) {
			$this->deleteAllCache();
			exit('Cache_Plugin::log(delete all cache.)');
		}

		if (! empty($this->log)) {
			$this->putContents($filename, serialize($this->log));

			foreach ($this->log as $line) {
				list($page, $type) = explode(' => ', $line);
				$_filename = $this->getFileName($page, 'log', $type);
				$array = array();
				if (file_exists($_filename)) {
					$array += $this->unserializeFile($_filename);
				}
				$array[$this->page] = '';
				$this->putContents($_filename, serialize($array));
			}
		}
	}
}


// [EOF] 223