<?php
// $Id: class.read_all.php,v 1.0.0 2006/07/08 00:00:00 TK Exp $


require_once CONVERT_CACHE_LIB_DIR . 'class.read.php';


class Convert_Cache_Read_All extends Convert_Cache_Read
{
	var $page;

	var $mode;

	var $config;


	function & getInstance()
	{
		static $instance = null;

		if ($instance === null) {
			$instance = new Convert_Cache_Read_All();
		}
		return $instance;
	}

	function run()
	{
		$this->setDefault();
		$this->setPageName();
		if ($this->page === false) {
			$this->mode = 'basic';
		} else {
			$this->setItemsDefault();
			$this->setMode();
			if ($this->mode == 'read') {
				$this->output();
			}
		}
	}

	function setDefault()
	{
		$this->page = '';
		$this->mode = 'no';
		$this->config = true;
	}

	function setMode()
	{
		if ($this->canWork()) {
			if (! $this->isBasicCache()) {
				$this->mode = 'basic';
			} elseif ($this->isPageCache($this->page)) {
				$this->mode = 'read';
			} elseif (! $this->isNoCache($this->page)) {
				$this->mode = 'make';
			}
		}
	}

	function canWork()
	{
		if ($this->isRequest()) {
			return false;
		}
		if ($this->isMobile()) {
			return false;
		}
		return true;
	}

	function isRequest()
	{
		$get  = input_filter_copy($_GET);
		if (isset($get['cmd']) || isset($get['plugin'])) {
			return true;
		}
		if (CONVERT_CACHE_POST) {
			return true;
		}
		return false;
	}

	function isMobile()
	{
		if (! isset($_SERVER['HTTP_USER_AGENT'])) {
			return false;
		}

		foreach ($this->getAgentsArray() as $agent) {
			if (preg_match($agent['pattern'], $_SERVER['HTTP_USER_AGENT'])) {
				return $agent['profile'] == 'keitai';
			}
		}
	}

	function getAgentsArray()
	{
		$filename = CONVERT_CACHE_DIR . 'agents.ini';
		if (! file_exists($filename)) {
			$agents = $this->makeAgentsCache($filename);
		} else {
			$agents = $this->unserializeFile($filename);
		}
		return $agents;
	}

	function makeAgentsCache($filename)
	{
		$ini = $this->getContents(DATA_HOME . 'pukiwiki.ini.php');
		$matches = array();
		if (preg_match('/\$agents = array\(.+\);/s', $ini, $matches)) {
			eval($matches[0]);
		} else {
			$agents = array(
				array('pattern' => '#.*#', 'profile' => 'keitai')
			);
		}
		$this->putContents($filename, serialize($agents));
		return $agents;
	}

	function isBasicCache()
	{
		$filenames = array(
			CONVERT_CACHE_DIR . 'def.ini',
			CONVERT_CACHE_DIR . 'var.ini',

			CONVERT_CACHE_DIR . 'menu.cfg',
			CONVERT_CACHE_DIR . 'plugin.cfg',

			CONVERT_CACHE_DIR . 'defaultpage',
		);
		return $this->existFiles($filenames);
	}

	function setPageName()
	{
		$arg = $this->getPageName();
		if ($arg == '') {
			$filename = CONVERT_CACHE_DIR . 'defaultpage';
			if (! file_exists($filename)) {
				$this->page = false;
				return;
			}
			$this->page = $this->getContents($filename);
		} else {
			$this->page = rawurldecode(input_filter_copy($arg));
		}
	}

	function getPageName()
	{
		$arg = '';
		if (isset($_SERVER['QUERY_STRING']) && $_SERVER['QUERY_STRING']) {
			$arg = $_SERVER['QUERY_STRING'];
		} elseif (isset($_SERVER['argv']) && ! empty($_SERVER['argv'])) {
			$arg = $_SERVER['argv'][0];
		}
		return $arg;
	}

	function setItemsDefault()
	{
		require_once CONVERT_CACHE_LIB_DIR . 'class.item.php';
		$obj =& Convert_Cache_Item::getInstance();
		$obj->setDefault($this->page);
	}

	function output()
	{
		global $_LANG, $_LINK, $_IMAGE;

		$filename = CONVERT_CACHE_DIR . 'def.ini';
		foreach ($this->unserializeFile($filename) as $key => $value) {
			define($key, $value);
		}

		mb_language(MB_LANGUAGE);
		mb_internal_encoding(SOURCE_ENCODING);
		ini_set('mbstring.http_input', 'pass');
		mb_http_output('pass');
		mb_detect_order('auto');

		$filenames = array(
				CONVERT_CACHE_DIR . 'var.ini',
				$this->getFileName($this->page, 'page'),
			);
		foreach ($filenames as $filename) {
			foreach ($this->unserializeFile($filename) as $key => $value) {
				$$key = $value;
			}
		}

		require_once CONVERT_CACHE_LIB_DIR . 'func.skin.php';

		$link = '(<a href="http://hack.tv/" title="cache">cache</a>)';
		$taketime = $link . microtime_float('full');
		require_once SKIN_FILE;
		exit;
	}

	// {{{ @skin

	function mergeMenu($menu)
	{
		if ($this->page === false) {
			return $menu;
		}
		if (! file_exists(CONVERT_CACHE_DIR . 'menu.cfg')) {
			$this->makeConfigCache(array('menu'));
		}
		if (! file_exists(CONVERT_CACHE_DIR . 'items')) {
			$this->makeItemsCache();
		}
		if (! $this->config) {
			return $menu;
		}
		$config = $this->unserializeFile(CONVERT_CACHE_DIR . 'menu.cfg');
		$items  = $this->unserializeFile(CONVERT_CACHE_DIR . 'items');
		$return_menu = '';
		foreach ($config as $array) {
			list($name) = $array;
			$name = strtolower($name);
			if ($name == 'menu') {
				$return_menu .= $menu;
			} else {
				if (isset($items[$name])) {
					$return_menu .= $items[$name];
				} else {
					$return_menu .= $this->getItems(ucfirst($name));
				}
			}
		}
		return $return_menu;
	}

	function makeItemsCache()
	{
		$obj =& Convert_Cache_Item::getInstance();
		$obj->make();
	}

	function getItems($name)
	{
		static $obj = null;

		if ($obj === null) {
			$obj = Convert_Cache_Item::getInstance();
		}
		$function = 'get' . $name;
		if (method_exists($obj, $function)) {
			return $obj->$function();
		}
	}

	function parseINI($basename)
	{
		$filename = CONVERT_CACHE_LIB_DIR . $basename;
		foreach (explode("\n", $this->getContents($filename)) as $value) {
			$value = trim($value, "\r\n");
			if ($value == '' || $value{0} == ';') {
				continue;
			}
			$return_values[] = $value;
		}
		return $return_values;
	}

	function makeBasicCache($array)
	{
		global $defaultpage;

		foreach ($array as $key => $value) {
			$filename = CONVERT_CACHE_DIR . $key . '.ini';
			$this->putContents($filename,  serialize($value));
		}
		$this->makeConfigCache(array('menu', 'plugin'));
		$this->putContents(CONVERT_CACHE_DIR . 'defaultpage', $defaultpage);
	}

	function removeBodyStatus($body)
	{
		$regex = '#<!-- convert_cache -->.+<!-- /convert_cache -->#s';
		return preg_replace($regex, '', $body);
	}

	// }}} @skin
}


function input_filter_copy($param)
{
	static $magic_quotes_gpc = null;

	if ($magic_quotes_gpc === null) {
	    $magic_quotes_gpc = get_magic_quotes_gpc();
	}

	if (is_array($param)) {
		return array_map('input_filter_copy', $param);
	} else {
		$result = str_replace("\0", '', $param);
		if ($magic_quotes_gpc) {
			$result = stripslashes($result);
		}
		return $result;
	}
}


// [EOF] 313